#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>

#include "tr.h"
#include "msd.h"
#include "db.h"
#include "stock.h"

typedef struct Flags { 
    unsigned int verbose : 4 ; 
} Flags ;

static void
usage ()
{
    cbanner ( 0, 
	     "[-t tolerance] [-v] db", 
	     "BRTT example", 
	     "BRTT", 
	     "support@brtt.com" ) ; 
    exit (1);
}

void
show_data ( Msd *msd ) 
{
    long i ; 
    static int *data ; 
    long npts = 0 ; 
    if ( umsd ( msd, &data, &npts ) ) { 
	complain ( 0, "umsd failed" ) ; 
    } else { 
	printf ( "    " ) ; 
	for ( i=0 ; i<npts ; i++ ) { 
	    printf ( " %8d", data[i] ) ; 
	    if ( ((i+1) % 5) == 0 ) { 
		printf ( "\n    " ) ; 
	    }
	}
	if ( ((i) % 5) != 0 ) {
	    printf ( "\n" ) ; 
	}
    }
}

void
show_msd ( Msd *msd, Flags flags ) 
{
    char net[16], sta[16], chan[16], loc[16] ; 
    char filename[FILENAME_MAX] ; 
    double samprate, time ; 
    int nsamp, has_sdh, has_b100, has_b1000 ;
    int dataformat, fpr, fdf, sparc_order, activity, io, 
    	level, quality, samprate_flags, record_size, correction ; 
    long offset ;
    char *s ;

    msdget ( msd, 
	MSD_NET, net, 
    	MSD_STA, sta, 
	MSD_CHAN, chan, 
	MSD_LOC, loc, 
	MSD_SAMPRATE, &samprate, 
	MSD_TIME, &time, 
	MSD_NSAMP, &nsamp, 
	MSD_HAS_SEED_HEADER, &has_sdh, 
	MSD_HAS_B100, &has_b100, 
	MSD_HAS_B1000, &has_b1000, 
	MSD_LEVEL, &level,
	MSD_DATAFORMAT, &dataformat, 
	MSD_FRAMES_PER_RECORD, &fpr, 
	MSD_FIRST_DATA_FRAME, &fdf, 
	MSD_SPARC_ORDER, &sparc_order, 
	MSD_FILENAME, filename, 
	MSD_OFFSET, &offset,
	MSD_ACTIVITY_FLAGS, &activity, 
	MSD_IO_FLAGS, &io, 
	MSD_QUALITY_FLAGS, &quality, 
	MSD_SAMPRATE_FLAGS, &samprate_flags, 
	MSD_RECORD_SIZE, &record_size, 
	MSD_TIME_CORRECTION, &correction, 
         0 ) ; 

    printf ( "\n%2s %5s %3s %2s  %s  %5d %11.7f %s 0x%lx\n", 
    		net, sta, chan, loc, s=strydtime(time), nsamp, samprate, filename, offset ) ; 
    free(s) ; 
    printf ( "    %5d b/rcd %2d fpr %2d fdf (%s) %s%s\n", record_size, fpr, fdf, 
		sparc_order ? "sparc" : "intel", 
    		has_b1000 ? " B1000" : "",
    		has_b100 ? " B100" : "" ) ; 
    printf ( "    dataformat %2d Steim level %d  \n", 
    	dataformat, level ) ; 
    printf ( "           io   activity quality samprate time-correction\n" ) ; 
    printf ( "  flags: %4x    %4x     %4x    %4x      %10d\n", 
    	io, activity, quality, samprate_flags, correction ) ;
    	
    if ( flags.verbose > 2) { 
	show_data ( msd ) ; 
    }
}

int
check_msd ( Dbptr db, Msd *msd, Flags flags ) 
{
    double time, samprate, tblk, sblk, tcalc ; 
    long totsamp, nsamp=0 ; 
    int nblk, seq, has_b100 ; 
    int shown_header = 0 ;
    char *s1, *s2, *s3 ;
    dbgetv(db, 0, "time", &time, 
    	"samprate", &samprate, 
	"nsamp", &totsamp, 
	NULL ) ;
    while ( msdnxt(msd) ) { 
	char net[16], sta[16], chan[16], loc[16] ; 

	if ( flags.verbose ) { 
	    show_msd ( msd, flags )  ;
	    shown_header = 1 ;
	}
	msdget ( msd, 
	    MSD_NET, net, 
	    MSD_STA, sta, 
	    MSD_CHAN, chan, 
	    MSD_LOC, loc, 
	    MSD_SAMPRATE, &sblk, 
	    MSD_TIME, &tblk, 
	    MSD_NSAMP, &nblk, 
	    MSD_SEQUENCE, &seq,
	    MSD_HAS_B100, &has_b100,
	     0 ) ; 

	if ( has_b100 && ! TRSAMERATE(sblk, samprate) ) { 
	    if ( ! shown_header ) { 
		show_msd ( msd, flags) ; 
		shown_header = 1 ;
	    }
	    fprintf ( stdout, "#%d: samprate doesn't match: %11.7f != %11.7f\n", 
	    	seq, samprate, sblk ) ; 
	}
	tcalc = SAMP2TIME(time,samprate,nsamp) ;
	if (flags.verbose) { 
	    fprintf(stdout, "%10ld %4d %s %s %s %8.3f\n", nsamp, nblk, s1=strtime(tblk), s2=strtime(tcalc), s3=strtdelta(tcalc-tblk), (tcalc-tblk)*samprate) ;
	    free(s1); 
	    free(s2); 
	    free(s3); 
	}
	if ( ! MSDSAMETIME(tblk, tcalc, samprate) ) { 
	    char *s1, *s2, *s3 ;
	    double tl, th ;
	    if ( ! shown_header ) { 
		show_msd ( msd, flags) ; 
		shown_header = 1 ;
	    }
	    fprintf ( stdout, "#%d: times don't match: \n\t%s\n\t%s\n\tt0=%s nsamp=%ld\n\tdelta=%.6f = %.3f dt\n",
	    	seq, 
		s1=strydtime(tblk), 
		s2=strydtime(tcalc), 
		s3=strydtime(time), nsamp,
		ABS(tcalc-tblk), ABS(tcalc-tblk)*samprate) ;
	    free(s1) ; 
	    free(s2) ;
	    free(s3) ;
	    if ( flags.verbose > 1) { 
		tl = SAMP2TIME(time, samprate, nsamp-trTOLERANCE) ; 
		th = SAMP2TIME(time, samprate, nsamp+trTOLERANCE) ; 
		fprintf ( stdout, "    %s\n    %s\n    %s\n", 
		    s1=strtime(tl), s2=strtime(tcalc), s3=strtime(th) ) ; 
		free(s1) ; 
		free(s2) ;
		free(s3) ;
	    }
	}
	nsamp += nblk ; 
	if ( nsamp >= totsamp ) { 
	    if ( nsamp > totsamp ) { 
		if ( ! shown_header ) { 
		    show_msd ( msd, flags) ; 
		    shown_header = 1 ;
		}
		fprintf(stdout, "nsamp doesn't match: wfdisc: %ld   miniseed: %ld", 
			totsamp, nsamp ) ; 
	    }
	    break ;
	}
    }
    return 0 ; 
}

int
check_record ( Dbptr db, Flags flags ) 
{
    Msd *msd ; 
    char path[FILENAME_MAX], *data ; 
    long size, offset ;

    msd = msdnew() ;
    switch ( dbextfile(db, 0, path) ) { 
	case 1:
	    size = mapfile ( path, 0,  (char **) &data ) ; 
	    dbgetv(db, 0, "foff", &offset, NULL ) ; 
	    msdput ( msd, MSD_FILENAME, path, 
			MSD_MSEED, data, 
			MSD_OFFSET, offset, 
			MSD_MSEED_NBYTES, size, 
			0 ) ;
	    if ( flags.verbose ) { 
		fprintf(stdout, "\n0x%10lx %s\n", offset, path) ;
	    }
	    check_msd(db, msd, flags ) ; 
	    break ;

	default:
	    complain(0, "Can't open %s", path ) ; 
	    break; 
    }
    msdfree(msd) ;
    return 0 ; 
}


int
main (int argc, char **argv)
{
    int             c,
		    errflg = 0;
    Flags           flags ;
    long		    nrecords ;
    char	   *dbname ;
    int		    ret ;
    Dbptr   	    db ;
    char	    datatype[4] ;
    double	    oldtolerance ;

    memset ( &flags, 0, sizeof(flags)) ;

    elog_init ( argc, argv ) ; 

    oldtolerance = trMSD_TOLERANCE ;
    while ((c = getopt (argc, argv, "t:vV")) != -1) {
	switch (c) {

	case 't':
	    trMSD_TOLERANCE = atof(optarg) ; 
	    elog_notify ( 0, "using tolerance of %.3f sample dt (instead of %.3f) for miniseed internal time tolerance", 
	    	trMSD_TOLERANCE, oldtolerance) ;
	    break ;

	case 'v':
	    flags.verbose++ ;
	    break;

	case 'V':
	    usage ();
	    break;

	case '?':
	    errflg++;
	    break ;
	}
    }

    if (errflg || argc-optind != 1)
	usage ();

    dbname = argv[optind++] ;

    ret = dbopen_database(dbname, "r", &db) ;
    if ( ret < 0 ) { 
	die ( 0, "Can't open %s", dbname ) ; 
    } else if ( ret == 0 ) { 
	db = dblookup (db, 0, "wfdisc", 0, 0) ; 
    }
    dbquery(db, dbRECORD_COUNT, &nrecords) ; 

    if ( nrecords < 1 ) { 
	die ( 0, "No records in %s", dbname ) ; 
    }

    for (db.record = 0 ; db.record < nrecords ; db.record++ ) { 
	char sta[16], chan[16] ; 
	double time, endtime, samprate ; 
	long nsamp ;
	dbgetv(db, 0,
	    "sta", sta,
	    "chan", chan, 
	    "time", &time, 
	    "nsamp", &nsamp, 
	    "endtime", &endtime, 
	    "samprate", &samprate, 
	    "datatype", datatype,
	    NULL ) ; 
	if ( flags.verbose ) { 
	    char *s1, *s2 ;
	    printf("%-5s %-3s %11.7f %8ld %s (%s)\n", 
	    	sta, chan, samprate, nsamp, 
		s1=strydtime(time),
		s2=strtdelta(nsamp/samprate) ) ; 
	    free(s1) ;
	    free(s2) ;
	}
	if ( strcmp(datatype, "sd") == 0 || strcmp(datatype, "MS") == 0 || strcmp(datatype, "S1") == 0 ) { 
	    check_record ( db, flags ) ; 
	} else if ( flags.verbose ) { 
	    printf("  ** not miniseed **\n" ) ; 
	}
    }
    return 0;
}
