#ifndef __elog__
#define __elog__
#include <stdarg.h>
#include <stdio.h>
#include <signal.h>


extern char *Program_Name ;

#define ELOG_NSEVERITY   7
#define ELOG_LOG 	0	/* library routines put messages on error log */
#define ELOG_NOTIFY 	1	/* informational messages */
#define ELOG_COMPLAIN 	2	/* program has problems, but will continue */
#define ELOG_DIE	3	/* program has problems and quits */
#define ELOG_FAULT	4	/* program has bug and dies from a fault */
#define ELOG_DEBUGGING	5	/* debugging level info ( -verbose messages) */
#define ELOG_ALERT	6	/* system level problem */

enum Elog_code { 
	ELOG_TBL,
	ELOG_TBLCP,
	ELOG_TBLCPCL,
	ELOG_ARGV,
	ELOG_ARGC,
	ELOG_COUNT,
	ELOG_VERSION,
	ELOG_MAXMSG,
	ELOG_TAG,
	ELOG_DELIVER,
	ELOG_DEBUG,
	ELOG_SIGNALS,
	ELOG_CALLBACK,
	ELOG_STRING
    } ;

typedef struct Elog_msg { 
    char *tag ; 	/* tag -- program name, pid, time, ... */
    char *msg ; 	/* msg from program */
    char *sys ;		/* system error message */
    int severity ;	/* severity level */
    int n ; 		/* # of repetitions */
} Elog_msg ;

#ifndef SIG2STR_MAX
#define SIG2STR_MAX 64
#endif

typedef struct Signal_info {
    int sig ; 
    char *name, *description ; 
} Signal_info ;

#define register_error       	elog_log
#define complain 		elog_complain
#define die     		elog_die
#define clear_register		elog_clear_register

#define STACKTRACE_TRIGGER "*stack*" 

#ifdef  __cplusplus
extern "C" {
#endif

extern Signal_info * signal_bysig ( int sig );
extern Signal_info * signal_byname ( char *name );
extern void signal_desc ( int sig, char *desc, int size );

extern void elog_msg_print ( FILE *file, Elog_msg *elog_msg );
extern void elog_print ( FILE *file, int first );
extern char * elog_string ( int first );
extern void elog_free_msg ( void *vmsg );
extern void elog_clear ( void );
extern int elog_init ( int argc, char **argv );
extern void elog_debugger ( void );
/*PRINTFLIKE3*/
extern char * elogstr ( int severity, int flag, char *fmt, ... ) __attribute__ ((format (printf, 3, 4))) ;
extern void elog ( int severity, int flag, char *fmt, va_list args ) ;
/*PRINTFLIKE2*/
extern void elog_log ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_notify ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_alert ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_debug ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_complain ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_die ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
/*PRINTFLIKE2*/
extern void elog_fault ( int flag, char *format, ... )__attribute__ ((format (printf, 2, 3))) ;
extern void elog_clear_register ( int deliver );
extern int elog_mark ( void );
extern void elog_truncate ( int n );
extern void elog_flush ( int deliver, int n );
extern int elog_query ( enum Elog_code code, int severity, void **ptrvoid );
extern int elog_set ( enum Elog_code code, int severity, void *value );

extern void setup_fault_handlers ( void fault_handler(int sig, siginfo_t *vsi, void *vuap), char *signals );

#ifdef	__cplusplus
}
#endif
#endif

/* $Id$ */
